/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzDisableVerbose_h
#include "ZzDisableVerbose.h"
#endif //ZzDisableVerbose_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC ZzSplitTask

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h


ZzSplitTask::ZzSplitTask()
{
	vernl(6,"ZzSplitTask("<< (void*)this << ")::ZzSplitTask()");
	fState = kSplitTaskNotStartedYet;
	fProgress = 0;
}

void ZzSplitTask::Cancel()
{
	ZzAssert(0);
}

bool ZzSplitTask::CanBeCancelled()
{
	return false;
}

bool ZzSplitTask::IsLengthy() const //$$$TODO replace this with a real value that would work for macro tasks. maybe.... ?
{
	return true;
}

bool ZzSplitTask::CanBeBackgrounded()
{
	return false;
}

void ZzSplitTask::SetProgress(float progress)
{
	fProgress = progress;
}

ArmeErr ZzSplitTask::PerformOnce()
{
	vernl(6,"ZzSplitTask("<< (void*)this << ")::PerformOnce()");
	ArmeErr err;
	return err;
}

void ZzSplitTask::DescChanged()
{
	fDescChanged = true;
}

bool ZzSplitTask::IsDescChanged()
{
	bool descChanged = fDescChanged;
	fDescChanged = false;
	return descChanged;
}

float ZzSplitTask::GetProgress() const
{
	vernl(6,"ZzSplitTask("<< (void*)this << ")::GetProgress() returning " << fProgress);
	return fProgress;
}

ArmeErr ZzSplitTask::_PerformNextBit()
{
	vernl(6,"ZzSplitTask("<< (void*)this << ")::_PerformNextBit() BEGINNING");
	ArmeErr err;
	// these assertions fire if the calling object neglects to
	// stop calling this method once the split task is finished or
	// in error.
	ZzAssert(fState != kSplitTaskFinished);
	ZzAssert(fState != kSplitTaskInError);
	if(fState == kSplitTaskNotStartedYet)
	{
		err = PerformOnce();
		if(err.type != kArmeOK)
		{
			vernl(0,"err.type != kArmeOK after PerformOnce() ------> ZzSplitTask("<< (void*)this << ")::_PerformNextBit() ENDING");
			fState = kSplitTaskInError;
			return err;
		}
		fState = kSplitTaskBeingPerformed;
	}
	err = PerformNextBit();
	if(err.type != kArmeOK)
	{
		vernl(0,"err.type != kArmeOK after PerformNextBit() ------> ZzSplitTask("<< (void*)this << ")::_PerformNextBit() IN ERROR");
		fState = kSplitTaskInError;
	}
	if(GetProgress() >= 1.0)
		fState = kSplitTaskFinished;
	vernl(6,"ZzSplitTask("<< (void*)this << ")::_PerformNextBit() ENDING");
	return err;
}

ZzSplitTask::~ZzSplitTask()
{
}

ArmeErr ZzSplitTask::ExecuteInProgressManagerContext(ZzSplitTask* splitTask, MIProgressBarDlgManager* progressMngr)
{
	ArmeErr err;
	err.type = kArmeOK;
	
	if(!splitTask)
		return err;
	float progress = 0.0;
	while(progress < 1.0)
	{
		err = splitTask->_PerformNextBit();
		if ( err.type != kArmeOK ) {
			delete splitTask;
			return err;
		}
		progress = splitTask->GetProgress();
		progressMngr->Update(progress);
	}
	delete splitTask;
	return err;
}

ZzMacroSplitTask::ZzMacroSplitTask()
{
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::ZzMacroSplitTask()");
	fCurTask = 0;
}

ZzMacroSplitTask::~ZzMacroSplitTask()
{
	for(SInt32 i = 0; i < fSubTasks.CountItems(); i++)
	{
		delete fSubTasks[i];
	}
}

void ZzMacroSplitTask::AdoptSubTask(ZzSplitTask* task)
{
	ZzAssert(GetState() == kSplitTaskNotStartedYet);
	ZzAssert(task->GetState() == kSplitTaskNotStartedYet);
	fSubTasks.AddItem(task);
}

ArmeErr ZzMacroSplitTask::PerformNextBit()
{
	ZzAssert(fCurTask < fSubTasks.CountItems());
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::PerformNextBit() BEGINNING");
	ArmeErr err;
	ZzAssert(GetState() != kSplitTaskCancelled);
	ZzAssert(GetState() != kSplitTaskFinished);
	if(fState == kSplitTaskNotStartedYet)
		fState = kSplitTaskBeingPerformed;
	err = fSubTasks[fCurTask]->_PerformNextBit();
	if(GetProgress() >= 1.0)
		fState = kSplitTaskFinished;
	if(fSubTasks[fCurTask]->GetProgress() >= 1.0)
		fCurTask++;
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::PerformNextBit() ENDING");
	return err;
}

void ZzMacroSplitTask::Cancel()
{
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::Cancel()");
	for(SInt32 i = 0; i < fSubTasks.CountItems(); i++)
	{
		fSubTasks[fCurTask]->Cancel();
	}
	fState = kSplitTaskCancelled;
}

const char* ZzMacroSplitTask::GetDesc() const
{	
	if(fCurTask == fSubTasks.CountItems())
		return "All tasks are finished";
	return fSubTasks[fCurTask]->GetDesc();
}

float ZzMacroSplitTask::GetProgress() const
{
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::GetProgress() BEGINNING");
	ZzAssert(GetState() != kSplitTaskCancelled);
	ZzAssert(fSubTasks.CountItems() > 0);
	if(fCurTask == fSubTasks.CountItems())
	{
		vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::GetProgress() fCurTask == fSubTasks.CountItems() --> Returning 1.0");
		return 1.0;
	}
	float amountPerSubTask = 1.0/fSubTasks.CountItems();
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::GetProgress() amountPerSubTask = " << amountPerSubTask);
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::GetProgress() fCurTask = " << fCurTask);
	float curSubTaskProgress = fSubTasks[fCurTask]->GetProgress();
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::GetProgress() curSubTaskProgress = " << curSubTaskProgress);
	float progress = amountPerSubTask*((float)fCurTask + curSubTaskProgress);
	vernl(6,"ZzMacroSplitTask("<< (void*)this << ")::GetProgress() RETURNING " << progress);
	return progress;
}

SInt32 ZzMacroSplitTask::GetNbSubTasks() const
{
	return fSubTasks.CountItems();
}

bool ZzMacroSplitTask::IsDescChanged()
{
	// fDescChanged is inited to true in subtasks so this
	// call will return true if we've just switched to a new task
	// or if the current tasks has called DescChanged();
	return fSubTasks[fCurTask]->IsDescChanged();
}

bool ZzMacroSplitTask::IsLengthy() const
{
	if(GetNbSubTasks() >= 10)
		return true;
	for(SInt32 i = 0; i < GetNbSubTasks(); i++)
	{
		if(fSubTasks[i]->IsLengthy())
			return true;
	}
	return false;
}



